@@ -1,4 +1,26 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
+/* NetworkManager -- Network link manager
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ *   Copyright © 2008 - 2009 Intel Corporation.
+ *
+ * Based on nm-openconnect-vpnc.c:
+ *   Copyright © 2005 - 2008 Red Hat, Inc.
+ *   Copyright © 2007 - 2008 Novell, Inc.
+ */
 
 #include <stdio.h>
 #include <string.h>
@@ -36,7 +58,6 @@ static const char *openconnect_binary_paths[] =
 };
 
 #define NM_OPENCONNECT_HELPER_PATH		LIBEXECDIR"/nm-openconnect-service-openconnect-helper"
-#define NM_OPENCONNECT_UDP_ENCAPSULATION_PORT	0 /* random port */
 
 typedef struct {
 	const char *name;
@@ -55,13 +76,18 @@ static ValidProperty valid_properties[] = {
 	{ NM_OPENCONNECT_KEY_PRIVKEY,     G_TYPE_STRING, 0, 0 },
 	{ NM_OPENCONNECT_KEY_CERTSIGS,    G_TYPE_STRING, 0, 0 },
 	{ NM_OPENCONNECT_KEY_LASTHOST,    G_TYPE_STRING, 0, 0 },
+	{ NM_OPENCONNECT_KEY_MTU,         G_TYPE_STRING, 0, 0 },
 	{ NM_OPENCONNECT_KEY_AUTOCONNECT, G_TYPE_BOOLEAN, 0, 0 },
+	{ NM_OPENCONNECT_KEY_PEM_PASSPHRASE_FSID, G_TYPE_BOOLEAN, 0, 0 },
+	{ NM_OPENCONNECT_KEY_PROXY,       G_TYPE_STRING, 0, 0 },
+	{ NM_OPENCONNECT_KEY_CSD_ENABLE,  G_TYPE_BOOLEAN, 0, 0 },
 	{ NULL,                           G_TYPE_NONE, 0, 0 }
 };
 
 static ValidProperty valid_secrets[] = {
 	{ NM_OPENCONNECT_KEY_COOKIE,  G_TYPE_STRING, 0, 0 },
 	{ NM_OPENCONNECT_KEY_GATEWAY, G_TYPE_STRING, 0, 0 },
+	{ NM_OPENCONNECT_KEY_GWCERT,  G_TYPE_STRING, 0, 0 },
 	{ NULL,                       G_TYPE_NONE, 0, 0 }
 };
 
@@ -133,7 +159,7 @@ validate_one_property (const char *key, const char *value, gpointer user_data)
 	}
 
 	/* Did not find the property from valid_properties or the type did not match */
-	if (!info->table[i].name) {
+	if (!info->table[i].name && strncmp(key, "form:", 5)) {
 		g_set_error (info->error,
 		             NM_VPN_PLUGIN_ERROR,
 		             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
@@ -236,7 +262,7 @@ nm_openconnect_start_openconnect_binary (NMOPENCONNECTPlugin *plugin,
 	GPtrArray *openconnect_argv;
 	GSource *openconnect_watch;
 	gint	stdin_fd;
-	const char *props_vpn_gw, *props_cookie, *props_cacert;
+	const char *props_vpn_gw, *props_cookie, *props_cacert, *props_mtu, *props_gwcert, *props_proxy;
 	
 	/* Find openconnect */
 	openconnect_binary = openconnect_binary_paths;
@@ -276,17 +302,34 @@ nm_openconnect_start_openconnect_binary (NMOPENCONNECTPlugin *plugin,
 		             "No WebVPN cookie provided.");
 		return -1;
 	}
+	props_gwcert = nm_setting_vpn_get_secret (s_vpn, NM_OPENCONNECT_KEY_GWCERT);
 
 	props_cacert = nm_setting_vpn_get_data_item (s_vpn, NM_OPENCONNECT_KEY_CACERT);
+	props_mtu = nm_setting_vpn_get_data_item (s_vpn, NM_OPENCONNECT_KEY_MTU);
+
+	props_proxy = nm_setting_vpn_get_data_item (s_vpn, NM_OPENCONNECT_KEY_PROXY);
 
 	openconnect_argv = g_ptr_array_new ();
 	g_ptr_array_add (openconnect_argv, (gpointer) (*openconnect_binary));
 
-	if (props_cacert && strlen(props_cacert)) {
+	if (props_gwcert && strlen(props_gwcert)) {
+		g_ptr_array_add (openconnect_argv, (gpointer) "--servercert");
+		g_ptr_array_add (openconnect_argv, (gpointer) props_gwcert);
+	} else if (props_cacert && strlen(props_cacert)) {
 		g_ptr_array_add (openconnect_argv, (gpointer) "--cafile");
 		g_ptr_array_add (openconnect_argv, (gpointer) props_cacert);
 	}
 
+	if (props_mtu && strlen(props_mtu)) {
+		g_ptr_array_add (openconnect_argv, (gpointer) "--mtu");
+		g_ptr_array_add (openconnect_argv, (gpointer) props_mtu);
+	}
+
+	if (props_proxy && strlen(props_proxy)) {
+		g_ptr_array_add (openconnect_argv, (gpointer) "--proxy");
+		g_ptr_array_add (openconnect_argv, (gpointer) props_proxy);
+	}
+		
 	g_ptr_array_add (openconnect_argv, (gpointer) "--syslog");
 	g_ptr_array_add (openconnect_argv, (gpointer) "--cookie-on-stdin");
 
@@ -337,8 +380,6 @@ real_connect (NMVPNPlugin   *plugin,
 {
 	NMSettingVPN *s_vpn;
 	gint openconnect_fd = -1;
-	gboolean success = FALSE;
-	GHashTable *secrets;
 
 	s_vpn = NM_SETTING_VPN (nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN));
 	g_assert (s_vpn);
@@ -387,6 +428,10 @@ real_need_secrets (NMVPNPlugin *plugin,
 		*setting_name = NM_SETTING_VPN_SETTING_NAME;
 		return TRUE;
 	}
+	if (!nm_setting_vpn_get_data_item (s_vpn, NM_OPENCONNECT_KEY_GWCERT)) {
+		*setting_name = NM_SETTING_VPN_SETTING_NAME;
+		return TRUE;
+	}
 	return FALSE;
 }
 
@@ -453,7 +498,8 @@ quit_mainloop (NMOPENCONNECTPlugin *plugin, gpointer user_data)
 	g_main_loop_quit ((GMainLoop *) user_data);
 }
 
-void create_persistent_tundev(void)
+static void
+create_persistent_tundev(void)
 {
 	struct passwd *pw;
 	struct ifreq ifr;
@@ -501,7 +547,8 @@ void create_persistent_tundev(void)
 	close(fd);
 }
 
-void destroy_persistent_tundev(void)
+static void
+destroy_persistent_tundev(void)
 {
 	struct ifreq ifr;
 	int fd;
